package  
{
	import flash.display.MovieClip;
	import flash.geom.Vector3D;
	
	public class Boid extends MovieClip
	{
		public static const MAX_FORCE 			:Number = 1.5;
		public static const MAX_VELOCITY 		:Number = 3;
		public static const DECISION_INTERVAL 	:Number = 2000; // in milliseconds
		
		public var position 	:Vector3D;
		public var velocity 	:Vector3D;
		public var target 		:Vector3D;
		public var desired 		:Vector3D;
		public var steering 	:Vector3D;
		public var mass			:Number;
		public var nextDecision	:Number;
		
		public function Boid(posX :Number, posY :Number, totalMass :Number = 20) {
			position 		= new Vector3D(posX, posY);
			velocity 		= new Vector3D(-1, -2);
			target	 		= new Vector3D(310, 240);
			desired	 		= new Vector3D(0, 0); 
			steering 		= new Vector3D(0, 0); 
			mass	 		= totalMass;
			nextDecision 	= 0; 
			
			truncate(velocity, MAX_VELOCITY);
			
			x = position.x;
			y = position.y;
			
			graphics.lineStyle(2, 0xffaabb);
			graphics.beginFill(0xFF0000);
			graphics.moveTo(0, 0);
			graphics.lineTo(0, -20);
			graphics.lineTo(10, 20);
			graphics.lineTo(-10, 20);
			graphics.lineTo(0, -20);
			graphics.endFill();
			
			graphics.moveTo(0, 0);
		}
		
		private function seek(target :Vector3D) :Vector3D {
			var force :Vector3D;
			
			desired = target.subtract(position);
			desired.normalize();
			desired.scaleBy(MAX_VELOCITY);
			
			force = desired.subtract(velocity);
			
			return force;
		}
		
		private function wander() :Vector3D {
			var now :Number = (new Date()).getTime();
			
			if (now >= nextDecision) {
				nextDecision = now + DECISION_INTERVAL;
				
				target.x = 20 + Game.width * Math.random() * 0.8;
				target.y = 20 + Game.height * Math.random() * 0.8;
			}
			
			return seek(target);
		}
		
		public function getAngle(vector :Vector3D) :Number {
			return Math.atan2(vector.y, vector.x);
		}
		
		public function truncate(vector :Vector3D, max :Number) :void {
			var i :Number;

			i = max / vector.length;
			i = i < 1.0 ? i : 1.0;
			
			vector.scaleBy(i);
		}
		
		public function update():void {
			steering = wander();
			
			truncate(steering, MAX_FORCE);
			steering.scaleBy(1 / mass);
			
			velocity = velocity.add(steering);
			truncate(velocity, MAX_VELOCITY);
			
			position = position.add(velocity);
			
			x = position.x;
			y = position.y;
			
			// Adjust boid rodation to match the velocity vector.
			rotation = 90 + (180 * getAngle(velocity)) / Math.PI;
			
			if (x < 0 || x > stage.stageWidth || y < 0 || y > stage.stageHeight) {
				reset();
			}
		}
		
		public function reset() :void {
			position.x = Game.width / 2;
			position.y = Game.height / 2;
			
			velocity.x = -1 * (Math.random() < 0.5 ? -2 : 1);
			velocity.y = -1 * (Math.random() < 0.5 ? -2 : 1);
			
			truncate(velocity, MAX_VELOCITY * 0.5);
			desired.x = desired.y = 0;
			steering.x = steering.y = 0;
		}
	}
}